%
% This file implements the method described in:
%
% "Robust Region Detection via Consensus Segmentation of Deformable Shapes"
% E. Rodola, S. Rota Bulo, and D. Cremers
% Computer Graphics Forum, 33(5), 2014
%
% If you use this code, please cite the paper above.
% 
% Emanuele Rodola (c) 2014
%
function [segM, time_ensemble, time_consensus] = consensus_segment(M, params)

% this should be 1 on well-behaving shapes
first_eigen = find(M.evals<params.zero_eigen, 1, 'last');

if first_eigen~=1
    warning('consensus_segment(): Skipping the first %d eigenvalues',first_eigen);
end

% compute descriptors on triangles
GPS = M.proj * ( M.evecs(:,1+first_eigen:end) ./ repmat(sqrt(M.evals(1+first_eigen:end)'), M.n, 1) );

% clusterize descriptors and accumulate pairwise co-occurrences
tic;
keys = create_ensemble(GPS, M.S_tri./max(M.S_tri), params);
time_ensemble = toc;
fprintf('Ensemble creation took %.2f seconds\n', time_ensemble);

% show every segmentation in the ensemble
% for i=1:params.consensus_n_segmentations
%     figure()
%     plot_scalar_map_tri(M, keys(:,i));
% end

[segM_, time_consensus, iters] = hierarchical_consensus(M, keys, params);

% separate into connected components and filter out small areas due to
% mesh artifacts

segM = zeros(1,M.m);

for s=1:max(segM_)
    [components,nc,~] = conn_comp(M.VERT', M.TRIV', double(segM_==s));
    
    % remove small segments
    newc = 0;
    for c=1:nc
        ind = (components==c);
        seg_area = sum(M.S_tri(ind));
        if seg_area < params.min_area % filter out
            components(ind) = 0;
            if newc==0
                newc = c;
            end
        else % keep and rename
            if newc>0 && c~=newc
                components(ind) = newc;
                newc = newc + 1;
            end
        end
    end
    
    ind = components>0;
    components(ind) = max(segM) + components(ind);
    segM = segM + components';
end

fprintf('Regions: %d\nElapsed: %.2f sec\nIterations: %d\n',max(segM),time_consensus,iters);

end
